@@ -2,7 +2,7 @@
 
 import os
 import urllib.request, urllib.parse, urllib.error
-from gi.repository import Gdk, GdkPixbuf, Gtk, GObject
+from gi.repository import Gdk, GdkPixbuf, GLib, Gtk, GObject
 import PIL.Image as Image
 import PIL.ImageDraw as ImageDraw
 
@@ -18,6 +18,7 @@ from mcomix import log
 from mcomix import message_dialog
 from mcomix import tools
 from mcomix.library.pixbuf_cache import get_pixbuf_cache
+from mcomix.i18n import _
 
 _dialog = None
 
@@ -267,7 +268,7 @@ class _BookArea(Gtk.ScrolledWindow):
                 return
 
             # If the current view is filtered, only draw new books that match the filter
-            if not (self._library.filter_string and 
+            if not (self._library.filter_string and
                     self._library.filter_string.lower() not in book.name.lower()):
                 self.add_books([book])
 
@@ -358,16 +359,16 @@ class _BookArea(Gtk.ScrolledWindow):
         elif path2 is None:
             return -1
 
-        name1 = os.path.split(path1.decode('utf-8'))[1].lower()
-        name2 = os.path.split(path2.decode('utf-8'))[1].lower()
+        name1 = os.path.split(path1)[1].lower()
+        name2 = os.path.split(path2)[1].lower()
 
-        return tools.alphanumeric_compare(name1, name2)
+        return tools.cmp(tools.AlphanumericSortKey(name1), tools.AlphanumericSortKey(name2))
 
     def _sort_by_path(self, treemodel, iter1, iter2, user_data):
         """ Compares two books based on their full path, in natural order. """
         path1 = self._liststore.get_value(iter1, 2)
         path2 = self._liststore.get_value(iter2, 2)
-        return tools.alphanumeric_compare(path1, path2)
+        return tools.cmp(tools.AlphanumericSortKey(path1), tools.AlphanumericSortKey(path2))
 
     def _icon_added(self, model, path, iter, *args):
         """ Justifies the alignment of all cell renderers when new data is
@@ -421,7 +422,7 @@ class _BookArea(Gtk.ScrolledWindow):
         if prefs['library cover size'] != old_size:
             self._cache.invalidate_all()
             collection = self._library.collection_area.get_current_collection()
-            GObject.idle_add(self.display_covers, collection)
+            GLib.idle_add(self.display_covers, collection)
 
     def _pixbuf_size(self, border_size=_BORDER_SIZE):
         # Don't forget the extra pixels for the border!
