@@ -37,6 +37,10 @@ from mcomix import tools
 from mcomix import box
 from mcomix import layout
 from mcomix import log
+from mcomix.transform import Matrix, Transform
+from mcomix.i18n import _
+
+from typing import List
 
 
 class MainWindow(Gtk.Window):
@@ -64,6 +68,7 @@ class MainWindow(Gtk.Window):
         self._last_scroll_destination = constants.SCROLL_TO_START
 
         self.layout = layout.create_dummy_layout()
+        self.transforms: List[Matrix] = []
         self._spacing = prefs['space between two pages']
         self._waiting_for_redraw = False
 
@@ -135,9 +140,9 @@ class MainWindow(Gtk.Window):
 
         table.attach(self._event_box, 1, 2, 2, 3, Gtk.AttachOptions.FILL|Gtk.AttachOptions.EXPAND,
             Gtk.AttachOptions.FILL|Gtk.AttachOptions.EXPAND, 0, 0)
-        table.attach(self._scroll[constants.HEIGHT_AXIS], 2, 3, 2, 3, Gtk.AttachOptions.FILL|Gtk.AttachOptions.SHRINK,
+        table.attach(self._scroll[constants.PageAxis.HEIGHT], 2, 3, 2, 3, Gtk.AttachOptions.FILL|Gtk.AttachOptions.SHRINK,
             Gtk.AttachOptions.FILL|Gtk.AttachOptions.SHRINK, 0, 0)
-        table.attach(self._scroll[constants.WIDTH_AXIS], 1, 2, 4, 5, Gtk.AttachOptions.FILL|Gtk.AttachOptions.SHRINK,
+        table.attach(self._scroll[constants.PageAxis.WIDTH], 1, 2, 4, 5, Gtk.AttachOptions.FILL|Gtk.AttachOptions.SHRINK,
             Gtk.AttachOptions.FILL, 0, 0)
         table.attach(self.menubar, 0, 3, 0, 1, Gtk.AttachOptions.FILL|Gtk.AttachOptions.SHRINK,
             Gtk.AttachOptions.FILL, 0, 0)
@@ -154,11 +159,11 @@ class MainWindow(Gtk.Window):
 
         # Determine zoom mode. If zoom_mode is passed, it overrides
         # the zoom mode preference.
-        zoom_actions = { constants.ZOOM_MODE_BEST : 'best_fit_mode',
-                constants.ZOOM_MODE_WIDTH : 'fit_width_mode',
-                constants.ZOOM_MODE_HEIGHT : 'fit_height_mode',
-                constants.ZOOM_MODE_SIZE : 'fit_size_mode',
-                constants.ZOOM_MODE_MANUAL : 'fit_manual_mode' }
+        zoom_actions = { constants.ZoomMode.BEST: 'best_fit_mode',
+                constants.ZoomMode.WIDTH: 'fit_width_mode',
+                constants.ZoomMode.HEIGHT: 'fit_height_mode',
+                constants.ZoomMode.SIZE: 'fit_size_mode',
+                constants.ZoomMode.MANUAL: 'fit_manual_mode' }
 
         if zoom_mode is not None:
             zoom_action = zoom_actions[zoom_mode]
@@ -201,12 +206,12 @@ class MainWindow(Gtk.Window):
 
         # Each "toggle" widget "eats" part of the main layout visible area.
         self._toggle_axis = {
-            self.thumbnailsidebar              : constants.WIDTH_AXIS ,
-            self._scroll[constants.HEIGHT_AXIS]: constants.WIDTH_AXIS ,
-            self._scroll[constants.WIDTH_AXIS] : constants.HEIGHT_AXIS,
-            self.statusbar                     : constants.HEIGHT_AXIS,
-            self.toolbar                       : constants.HEIGHT_AXIS,
-            self.menubar                       : constants.HEIGHT_AXIS,
+            self.thumbnailsidebar              : constants.PageAxis.WIDTH,
+            self._scroll[constants.PageAxis.HEIGHT]: constants.PageAxis.WIDTH,
+            self._scroll[constants.PageAxis.WIDTH] : constants.PageAxis.HEIGHT,
+            self.statusbar                     : constants.PageAxis.HEIGHT,
+            self.toolbar                       : constants.PageAxis.HEIGHT,
+            self.menubar                       : constants.PageAxis.HEIGHT,
         }
 
         # Start with all "toggle" widgets hidden to avoid ugly transitions.
@@ -266,6 +271,7 @@ class MainWindow(Gtk.Window):
             toggleaction = self.actiongroup.get_action('fullscreen')
             toggleaction.set_active(True)
 
+
         if prefs['previous quit was quit and save']:
             fileinfo = self.filehandler.read_fileinfo_file()
 
@@ -321,8 +327,8 @@ class MainWindow(Gtk.Window):
         # FIXME: what if scroll_to is different?
         if not self._waiting_for_redraw:  # Don't stack up redraws.
             self._waiting_for_redraw = True
-            GObject.idle_add(self._draw_image, scroll_to,
-                             priority=GObject.PRIORITY_HIGH_IDLE)
+            GLib.idle_add(self._draw_image, scroll_to,
+                             priority=GLib.PRIORITY_HIGH_IDLE)
 
     def _update_toggle_preference(self, preference, toggleaction):
         ''' Update "toggle" widget corresponding <preference>.
@@ -402,29 +408,29 @@ class MainWindow(Gtk.Window):
                 rotation_list = [0] * len(pixbuf_list)
             virtual_size = [0, 0]
             for i in range(pixbuf_count):
-                if rotation_list[i] in (90, 270):
+                if tools.rotation_swaps_axes(rotation_list[i]):
                     size_list[i].reverse()
                 size = size_list[i]
                 virtual_size[distribution_axis] += size[distribution_axis]
                 virtual_size[alignment_axis] = max(virtual_size[alignment_axis],
                                                    size[alignment_axis])
-            rotation = image_tools.get_size_rotation(*virtual_size)
-            rotation = (rotation + prefs['rotation']) % 360
-            if rotation in (90, 270):
+            rotation = tools.compile_rotations(
+                image_tools.get_size_rotation(*virtual_size), prefs['rotation'])
+            if tools.rotation_swaps_axes(rotation):
                 distribution_axis, alignment_axis = alignment_axis, distribution_axis
                 orientation = list(orientation)
-                orientation.reverse()
+                orientation.reverse() # 2D only
                 for i in range(pixbuf_count):
                     if do_not_transform[i]:
                         continue
-                    size_list[i].reverse()
+                    size_list[i].reverse() # 2D only
             if rotation in (180, 270):
                 orientation = tools.vector_opposite(orientation)
             for i in range(pixbuf_count):
-                rotation_list[i] = (rotation_list[i] + rotation) % 360
-            if prefs['vertical flip'] and rotation in (90, 270):
+                rotation_list[i] = tools.compile_rotations(rotation_list[i], rotation)
+            if prefs['vertical flip'] and tools.rotation_swaps_axes(rotation):
                 orientation = tools.vector_opposite(orientation)
-            if prefs['horizontal flip'] and rotation in (0, 180):
+            if prefs['horizontal flip'] and not tools.rotation_swaps_axes(rotation):
                 orientation = tools.vector_opposite(orientation)
 
             self.layout = layout.FiniteLayout.create_finite_layout(
@@ -438,19 +444,23 @@ class MainWindow(Gtk.Window):
             content_boxes = self.layout.get_content_boxes()
             scaled_sizes = list(map(box.Box.get_size, content_boxes))
 
+            self.transforms = [Transform.ID] * pixbuf_count
             for i in range(pixbuf_count):
                 if do_not_transform[i]:
                     continue
                 pixbuf_list[i] = image_tools.fit_pixbuf_to_rectangle(
                     pixbuf_list[i], scaled_sizes[i], rotation_list[i])
+                self.transforms[i] += Transform.from_rotation(rotation_list[i]) # FIXME also include scales
 
             for i in range(pixbuf_count):
                 if do_not_transform[i]:
                     continue
                 if prefs['horizontal flip']:
-                    pixbuf_list[i] = pixbuf_list[i].flip(horizontal=True)
+                    pixbuf_list[i] = image_tools.flip_pixbuf(pixbuf_list[i], 0)
+                    self.transforms[i] += Transform.from_flips(True, False)
                 if prefs['vertical flip']:
-                    pixbuf_list[i] = pixbuf_list[i].flip(horizontal=False)
+                    pixbuf_list[i] = image_tools.flip_pixbuf(pixbuf_list[i], 1)
+                    self.transforms[i] += Transform.from_flips(False, True)
                 pixbuf_list[i] = self.enhancer.enhance(pixbuf_list[i])
 
             for i in range(pixbuf_count):
@@ -684,15 +694,15 @@ class MainWindow(Gtk.Window):
         pageselect.Pageselector(self)
 
     def rotate_90(self, *args):
-        prefs['rotation'] = (prefs['rotation'] + 90) % 360
+        prefs['rotation'] = tools.compile_rotations(prefs['rotation'], 90)
         self.draw_image()
 
     def rotate_180(self, *args):
-        prefs['rotation'] = (prefs['rotation'] + 180) % 360
+        prefs['rotation'] = tools.compile_rotations(prefs['rotation'], 180)
         self.draw_image()
 
     def rotate_270(self, *args):
-        prefs['rotation'] = (prefs['rotation'] + 270) % 360
+        prefs['rotation'] = tools.compile_rotations(prefs['rotation'], 270)
         self.draw_image()
 
     def flip_horizontally(self, *args):
@@ -726,7 +736,8 @@ class MainWindow(Gtk.Window):
         # Disable action until transition if complete.
         toggleaction.set_sensitive(False)
         if toggleaction.get_active():
-            self.save_window_geometry()
+            if self.previous_size != (None, None):
+                self.save_window_geometry()
             self.fullscreen()
         else:
             self.unfullscreen()
@@ -903,9 +914,9 @@ class MainWindow(Gtk.Window):
                 if widget.get_visible():
                     axis = self._toggle_axis[widget]
                     requisition = widget.size_request()
-                    if constants.WIDTH_AXIS == axis:
+                    if constants.PageAxis.WIDTH == axis:
                         size = requisition.width
-                    elif constants.HEIGHT_AXIS == axis:
+                    elif constants.PageAxis.HEIGHT == axis:
                         size = requisition.height
                     dimensions[axis] -= size
 
@@ -1074,8 +1085,9 @@ class MainWindow(Gtk.Window):
         else:
             text += '\n'
         page_number = self.imagehandler.get_current_page()
+        number_of_pages = self.imagehandler.get_number_of_pages()
         if page_number:
-            text += '%s %s' % (_('Page'), page_number)
+            text += '%s %d / %d' % (_('Page'), page_number, number_of_pages)
         text = text.strip('\n')
         if text:
             self.osd.show(text)
@@ -1101,25 +1113,27 @@ class MainWindow(Gtk.Window):
     def get_window_geometry(self):
         return self.get_position() + self.get_size()
 
-    def save_window_geometry(self):
-        (
-            prefs['window x'],
-            prefs['window y'],
-            prefs['window width'],
-            prefs['window height'],
-
-        ) = self.get_window_geometry()
+    def save_window_geometry(self) -> None:
+        x, y, width, height = self.get_window_geometry()
+        prefs['window x'] = x
+        prefs['window y'] = y
+        prefs['window width'] = width
+        prefs['window height'] = height
+        prefs['window maximized'] = self.is_maximized()
 
     def restore_window_geometry(self):
         if self.get_window_geometry() == (prefs['window x'],
                                           prefs['window y'],
                                           prefs['window width'],
-                                          prefs['window height']):
+                                          prefs['window height']) \
+           and self.is_maximized() == prefs['window maximized']:
             return False
-        self.resize(prefs['window width'], prefs['window height'])
-        # FIXME: Moving the Window on Windows causes it to go completely offscreen
-        if sys.platform != 'win32':
-            self.move(prefs['window x'], prefs['window y'])
+
+        self.move(prefs['window x'], prefs['window y'])
+        if prefs['window maximized']:
+            self.maximize()
+        else:
+            self.resize(prefs['window width'], prefs['window height'])
         return True
 
     def update_space(self):
@@ -1159,7 +1173,7 @@ class MainWindow(Gtk.Window):
 
         # This hack is to avoid Python issue #1856.
         for thread in threading.enumerate():
-            if thread is not threading.currentThread():
+            if thread is not threading.currentThread() and not isinstance(thread, threading._DummyThread):
                 log.debug('Waiting for thread %s to finish before exit', thread)
                 thread.join()
 
